#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentTypeDHLIntraship
# base         DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType
# state        public
#----------------------------------------------------------------------------------------
# description  Class for shipping fulfillment type connection
#========================================================================================
package DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentTypeDHLIntraship;
use base DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType;
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

use strict;

#========================================================================================
# function     featureName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $FeatureName = ShippingFulfillmentTypeDHLIntraship->featureName;
#----------------------------------------------------------------------------------------
# description   Returns the feature name 'DHLIntraship'.
#----------------------------------------------------------------------------------------
# return       $FeatureName | feature name (undef means no feature count) | String
#========================================================================================
sub featureName () { return 'DHLIntraship'; }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentTypeDHLIntraship = DE_EPAGES::DHLIntraship::API::Object::
# syntax       ShippingFulfillmentTypeDHLIntraship->insert($hInfo);
# example      my $Class = LoadClassByAlias('ShippingFulfillmentTypeDHLIntraship');
#               my $ShippingFulfillmentTypeDHLIntraship = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook
#               'OBJ_InsertShippingFulfillmentTypeDHLIntraship' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $ShippingFulfillmentTypeDHLIntraship | new object | object
# hook         OBJ_InsertShippingFulfillmentTypeDHLIntraship | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentTypeDHLIntrashipID - shipping label id - int
#               </ul>
#========================================================================================
sub insert {
    my ($Package, $hInfo) = @_;

    my $self = $Package->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertShippingFulfillmentTypeDHLIntraship',
        {'Object' => $self, 'ShippingFulfillmentTypeDHLIntrashipID' => $self->id });
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentTypeDHLIntraship->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook
#               'OBJ_DeleteShippingFulfillmentTypeDHLIntraship' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteShippingFulfillmentTypeDHLIntraship | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ShippingFulfillmentTypeDHLIntrashipID - shipping label id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;
    TriggerHook('OBJ_DeleteShippingFulfillmentTypeDHLIntraship',
        {'Object' => $self, 'ShippingFulfillmentTypeDHLIntrashipID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ShippingFulfillmentTypeDHLIntraship->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdate$ShippingFulfillmentTypeDHLIntraship'
#               and 'OBJ_AfterUpdate$ShippingFulfillmentTypeDHLIntraship'.
#----------------------------------------------------------------------------------------
# input        $hValues | hash containing values to set | hash
# input        $LanguageID | language code ID | string
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdate$ShippingFulfillmentTypeDHLIntraship | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>$ShippingFulfillmentTypeDHLIntrashipID - shipping label id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdate$ShippingFulfillmentTypeDHLIntraship | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>$ShippingFulfillmentTypeDHLIntrashipID - shipping label id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;
    TriggerHook(    'OBJ_BeforeUpdateShippingFulfillmentTypeDHLIntraship',
        {'Object'=>$self, 'Values'=> $hValues, 'ShippingFulfillmentTypeDHLIntrashipID' => $self->id,
            'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateShippingFulfillmentTypeDHLIntraship',
        {'Object'=>$self, 'Values'=> $hValues, 'ShippingFulfillmentTypeDHLIntrashipID' => $self->id,
            'LanguageID' => $LanguageID});
    return;
}


1;