#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Address
# state        public
# description  Address element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Address;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );

use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Origin;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Address = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Address->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Address | DHL Versenden address element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hAddress = $self->assemble($Address);
#----------------------------------------------------------------------------------------
# description  Create the hash with address data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $Address | Address object | object
# input        $Name2 | Second name (only for Receivers) | string
# return       $hAddress | DHL Versenden address element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($Address, $Name2) = @_;

    my $hAddressData = $Address->get( [ 'Street', 'HouseNumber', 'Street2', 'Zipcode', 'City' ] );

    my $hAddress = {};

    if (defined($Name2)) {
        $hAddress->{'name2'} = $Name2;
    }
    $hAddress->{'addressStreet'} = $hAddressData->{'Street'};
    $hAddress->{'addressAddition'} = $self->_concatenateAddressExtensionAndStreet2($hAddressData);
    $hAddress->{'postalCode'} = $hAddressData->{'Zipcode'};
    $hAddress->{'city'} = $hAddressData->{'City'};;


    my $hOrigin = $self->_assembleOrigin($Address);

    return {
                %{$hAddress},
                # no optional dispatchingInformation (not used)
                %{$hOrigin},
           };
}

#========================================================================================
# Local methods
#========================================================================================
sub _assembleOrigin {
    my $self = shift;
    my ($Address) = @_;

    return DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Origin->assemble($Address);
}

sub _concatenateAddressExtensionAndStreet2 {
    my $self = shift;
    my ($hAddressData) = @_;

    my @AddressAddition = ();

    if (isValid($hAddressData->{'AddressExtension'})) {
        push(@AddressAddition, $hAddressData->{'AddressExtension'});
    }

    if (isValid($hAddressData->{'Street2'})) {
        push(@AddressAddition, $hAddressData->{'Street2'});
    }

    return join(' ', @AddressAddition);
}

1;