#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Communication
# state        public
# description  Communication element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Communication;


use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Communication = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Communication->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Communication | DHL Versenden communication element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hCommunication = $self->assemble();
#----------------------------------------------------------------------------------------
# description  Create the hash with communication data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $Address | DHL Versenden address object | object
# return       $hCommunication | DHL Versenden communication item element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($Address) = @_;

    my $hAddressContactData = $Address->get( [ 'Phone', 'EMail' ] );
    my $hContactData;
    $hContactData->{'phone'} = $hAddressContactData->{'Phone'} if defined $hAddressContactData->{'Phone'};
    $hContactData->{'email'} = $hAddressContactData->{'EMail'} if defined $hAddressContactData->{'EMail'};

    return { %{$hContactData} };
}

1;