#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Name
# state        public
# description  Name element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Name;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );


use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Name = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Name->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Name | DHL Versenden name element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hName = $self->assemble($ShopAddress);
#----------------------------------------------------------------------------------------
# description  Create the hash with name data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShopAddress | shop address object | object
# return       $hName | DHL Versenden name element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($ShopAddress) = @_;

    my $hName = {};
    my $Company = $ShopAddress->get('Company');

    if (isValid($Company)) {
        $hName->{'name1'} =  $Company;
        $hName->{'name2'} =  $ShopAddress->get('FullName');

    } else {
        $hName->{'name1'} = $ShopAddress->get('FullName');
    }

    return { %{$hName} };
}

1;