#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Origin
# state        public
# description  Address Origin element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Origin;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Origin = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Origin->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Origin | DHL Versenden address origin element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hOrigin = $self->assemble();
#----------------------------------------------------------------------------------------
# description  Create the hash with address origin data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $Address | DHL Versenden address object | object
# return       $hOrigin | DHL Versenden address origin item element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($Address) = @_;

    my $hAddressData = $Address->get([ 'Country', 'State' ]);
    my $hAddressCountryData;
    $hAddressCountryData->{'country'} = $hAddressData->{'Country'}->{'Code3'};
    $hAddressCountryData->{'countryISOCode'} = $hAddressData->{'Country'}->{'Code2'};

    if (defined $hAddressData->{'State'}) {
        $hAddressCountryData->{'state'} = $hAddressData->{'State'};
    }

    return { %{$hAddressCountryData}};
}

1;