#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipment
# state        public
# description  Shipment element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipment;

use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::ShipmentDetails;
use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipper;
use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Receiver;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shipment = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipment->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Shipment | DHL Versenden shipment element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShipment = $self->assemble();
#----------------------------------------------------------------------------------------
# description  Create the hash with shipment data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL Versenden shipping label object | object
# return       $hShipment | DHL Versenden shipment element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $hShipmentDetails = $self->_assembleShipmentDetails($ShippingLabel);
    my $hShipper = $self->_assembleShipper($ShippingLabel);
    my $hReceiver = $self->_assembleReceiver($ShippingLabel);

    return {
            'shipments' => [{
                %{$hShipmentDetails},
                %{$hShipper},
                %{$hReceiver},
            }]
    };
}

#========================================================================================
# Local methods
#========================================================================================
sub _assembleShipmentDetails {
    my $self = shift;
    my ($ShippingLabel) = @_;

    return DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::ShipmentDetails->assemble($ShippingLabel);
}

sub _assembleShipper {
    my $self = shift;
    my ($ShippingLabel) = @_;

    return DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipper->assemble($ShippingLabel);
}

sub _assembleReceiver {
    my $self = shift;
    my ($ShippingLabel) = @_;

    return DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Receiver->assemble($ShippingLabel);
}

1;