#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::ShipmentOrder
# state        public
# description  ShipmentOrder element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::ShipmentOrder;

use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipment;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ShipmentOrder = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::ShipmentOrder->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $ShipmentOrder | DHL Versenden shipment order element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShipmentOrder = $self->assemble();
#----------------------------------------------------------------------------------------
# description  Create the hash with shipment order data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL Versenden shipping label object | object
# return       $hShipmentOrder | DHL Versenden shipment order element and warnings | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $hShipment = $self->_assembleShipment($ShippingLabel);

    return {
             'profile' => 'STANDARD_GRUPPENPROFIL',
             %{$hShipment},
           };
}

#========================================================================================
# Local methods
#========================================================================================
sub _assembleShipment {
    my $self = shift;
    my ($ShippingLabel) = @_;

    return DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipment->assemble($ShippingLabel);
}

1;