#========================================================================================
# package      DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipper
# state        public
# description  Shipper element for the DHL Versenden shipping label request builder
#========================================================================================
package DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipper;

use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Name;
use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Address;
use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Communication;

use strict;

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Shipper = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipper->new();
#----------------------------------------------------------------------------------------
# description  Object contructor.
#----------------------------------------------------------------------------------------
# return       $Shipper | DHL Versenden shipper element object | object
#========================================================================================
sub new {
    my $class = shift;

    my $self  = bless {}, $class;

    return $self;
}

#========================================================================================
# function     assemble
# state        public
#----------------------------------------------------------------------------------------
# syntax       $hShipper = $self->assemble($ShippingLabel);
#----------------------------------------------------------------------------------------
# description  Create the hash with shipper data for the Shipping label request
#----------------------------------------------------------------------------------------
# input        $ShippingLabel | DHL Versenden shipping label object | object
# return       $hShipper | DHL Versenden shipper element | ref.hash
#========================================================================================
sub assemble {
    my $self = shift;
    my ($ShippingLabel) = @_;

    my $Order = $ShippingLabel->get('Order');
    my $ShopAddress = $Order->get('ShopAddress');

    my $hName = $self->_assembleName($ShopAddress);
    my $hShipperAddress = $self->_assembleAddress($ShopAddress);
    my $hCommunication = $self->_assembleCommunication($ShopAddress);

    return {
            'shipper' => {
                %{$hName},
                %{$hShipperAddress},
                %{$hCommunication},
            }
        };
}

#========================================================================================
# Local methods
#========================================================================================
sub _assembleName {
    my $self = shift;
    my ($ShopAddress) = @_;

    return DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Name->assemble($ShopAddress);
}

sub _assembleAddress {
    my $self = shift;
    my ($ShopAddress) = @_;

    return DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Address->assemble($ShopAddress);
}

sub _assembleCommunication {
    my $self = shift;
    my ($ShopAddress) = @_;

    return DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Communication->assemble($ShopAddress);
}

1;