#========================================================================================
# package      DE_EPAGES::DHLIntraship::Hooks::Attribute
# state        private
#========================================================================================
package DE_EPAGES::DHLIntraship::Hooks::Attribute;

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath);
use DE_EPAGES::ThirdPartyConfig::API::Constants qw (ATTRIBUTES_CATEGORY_SHIPPING);

#========================================================================================
# function     OnGetThirdPartyConfigAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetThirdPartyConfigAttributes($hParams);
#----------------------------------------------------------------------------------------
# description  API_GetThirdPartyConfigAttributes triggered, register attributes to be
#               configured in PBO
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>AttributesCategory
#                   <li>LanguageID
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetThirdPartyConfigAttributes {
    my ($hParams) = @_;

    return unless $hParams->{'AttributesCategory'} eq ATTRIBUTES_CATEGORY_SHIPPING;
    my $ShippingFulfillmentType  = LoadObjectByPath('/ShippingFulfillmentTypes/DHLIntraship');

    push(@{$hParams->{'Attributes'}}, {
        'SectionID'   => $ShippingFulfillmentType->alias,
        'SectionName' => $ShippingFulfillmentType->get('NameOrAlias', $hParams->{'LanguageID'}),
        'Attributes' => [
            {'ObjectID' => $ShippingFulfillmentType->id, 'AttributeAlias' => 'Logging'},
            {'ObjectID' => $ShippingFulfillmentType->id, 'AttributeAlias' => 'WebserviceURL'},
            {'ObjectID' => $ShippingFulfillmentType->id, 'AttributeAlias' => 'WebserviceTestURL'},
            {'ObjectID' => $ShippingFulfillmentType->id, 'AttributeAlias' => 'RegistrationURL'},
            {'ObjectID' => $ShippingFulfillmentType->id, 'AttributeAlias' => 'AdminURL'},
            {'ObjectID' => $ShippingFulfillmentType->id, 'AttributeAlias' => 'DHLSandboxApiKey'},
            {'ObjectID' => $ShippingFulfillmentType->id, 'AttributeAlias' => 'DHLProductionApiKey'},
        ]
    });

    return;
}

sub SetName {
    my ($hParams) = @_;
    my $ShippingFulfillmentMethodDHLIntraship = $hParams->{'Object'};

    my $Shop = $ShippingFulfillmentMethodDHLIntraship->get('Shop');
    my $Languages = $Shop->get('Languages');
    foreach my $Language (@{$Languages}) {
        my $LanguageID = $Language->{LanguageID};
        $ShippingFulfillmentMethodDHLIntraship->set({'Name' => 'DHL Versenden'}, $LanguageID);
    }

    return;
}

1;
