#========================================================================================
# package      DE_EPAGES::DHLIntraship::UI::ShippingFulfillmentMethod
# state        private
#----------------------------------------------------------------------------------------
# description  User interface for ShippingFulfillmentMethodDHLIntraship,
#
#========================================================================================
package DE_EPAGES::DHLIntraship::UI::ShippingFulfillmentMethodDHLIntraship;
use base DE_EPAGES::Presentation::UI::Object;

use DE_EPAGES::DHLIntraship::API::Constants qw ( $GET_MANIFEST_OPERATION_IS_VALID );
use DE_EPAGES::DHLIntraship::API::Tools;
use DE_EPAGES::DHLIntraship::API::Object::RemoteRESTDHL;

use strict;


#========================================================================================
# function     ViewSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       ViewSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  View the settings of the shipping fulfillment method
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#========================================================================================
sub ViewSettings {
    my $self = shift;
    my ($Servlet) = @_;

    $self->SUPER::View($Servlet);
}

#========================================================================================
# function     SaveSettings
# state        public
#----------------------------------------------------------------------------------------
# syntax       SaveSettings( $Servlet );
#----------------------------------------------------------------------------------------
# description  Save the settings of the shipping fulfillment method
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#========================================================================================
sub SaveSettings {
    my $self = shift;
    my ($Servlet) = @_;

    my $ShippingFulfillmentMethod = $Servlet->object();
    my $Form = $Servlet->form();
    $Form->form($ShippingFulfillmentMethod, 'FulfillmentMethod');

    my $MethodName = (caller(0))[3];
    DE_EPAGES::DHLIntraship::API::Tools->LogShopContext(
            $ShippingFulfillmentMethod,
            $MethodName,
            sub {
                if ($self->_areValidCredentials($Form, $ShippingFulfillmentMethod)) {
                    $self->SUPER::Save($Servlet);
                } else {
                    $Form->executeFormError({
                        'Form'     => 'FulfillmentMethod',
                        'Name'     => 'InvalidCredentials',
                        'Reason'   => 'InvalidCredentials',
                    });
                }
                $self->SUPER::Save($Servlet);
            }
    );

    return;
}

#========================================================================================
# function     _areValidCredentials
# state        private
#----------------------------------------------------------------------------------------
# syntax       $areValid = $self->_areValidCredentials( $Form );
#----------------------------------------------------------------------------------------
# description  Validate provided credentials against web service
#----------------------------------------------------------------------------------------
# input        $Form | servlet object | object
# input        $ShippingFulfillmentMethod | $ShippingFulfillmentMethod object | object
# return       $areValid | credentials are successfully validated | boolean
#========================================================================================
sub _areValidCredentials {
    my $self = shift;
    my ($Form, $ShippingFulfillmentMethod) = @_;

    my $IsTestMode = $Form->value('IsInTestMode');
    my $EKP = $Form->value('EKP');
    my $TeilnehmerNational = $Form->value('TeilnehmerNational');
    my $TeilnehmerWeltPaket = $Form->value('TeilnehmerWeltpaket');

    my $userToTest = $Form->value('WebserviceUsername');
    my $passwordToTest;
    if ( $Form->value('WebservicePassword') =~ /^\*+$/) {
        $passwordToTest =  $ShippingFulfillmentMethod->get('WebservicePassword');
    } else {
        $passwordToTest = $Form->value('WebservicePassword');
    }

    my $aAuthorizationData = [$userToTest,$passwordToTest];
    # The DHLBillingnumber is a 14 digit string combining EKP+TeilnehmerNational+TeilnehmerWeltpaket.
    # It is needed for the orders validation request (/shipping/v2/orders?validate=true)
    my $DHLBillingNumber = $EKP.$TeilnehmerNational.$TeilnehmerWeltPaket;

    my $RemoteRESTDHL = DE_EPAGES::DHLIntraship::API::Object::RemoteRESTDHL->new();
    my $areValid = $RemoteRESTDHL->validateCredentials($DHLBillingNumber, $ShippingFulfillmentMethod, $aAuthorizationData, $IsTestMode);

    return $areValid;
}

1;
