#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::RemoteCredentialsValidation
# base         Test::Class
# state        private
#----------------------------------------------------------------------------------------
# description  Tests for getManifest SOAP call (integration tests)
#========================================================================================
package DE_EPAGES::DHLIntraship::t::RemoteCredentialsValidation;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( RunOnStore GetTestSite );
use DE_EPAGES::Object::API::Factory qw(LoadObjectByPath ExistsObjectByPath LoadRootObject);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );

use DE_EPAGES::DHLIntraship::API::Constants qw (
        $GET_MANIFEST_OPERATION_IS_VALID
        $GET_MANIFEST_OPERATION_IS_UNVALID
);
use DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest;
use DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationResponse;
use DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient;

use Test::More;
use Test::Exception;

use strict;

use constant TEST_EKP => '2222222222';
use constant TEST_NATIONAL_ACCOUNT_NUMBER => TEST_EKP . '0101';
use constant TEST_NATIONAL_ACCOUNT_NUMBER_WRONG => TEST_EKP . '0109';
use constant ROOTOBJECT => 'Test';
use constant ROOTPATH => '/' . ROOTOBJECT;
use constant SHOPPATH => ROOTPATH . '/TestShopDHLVersenden';
use constant IS_TEST_MODE => 1;
use constant WRONG_USER => 'user_wrong';
use constant WRONG_PASSWORD => 'password_wrong';

sub Init {

    return if ExistsObjectByPath(ROOTPATH) && $ENV{'SKIP_CLEANUP'};

    Cleanup();

    my $System = LoadRootObject();
    $System->insertChild( 'Object', { Alias => ROOTOBJECT } );
    note 'importing shop ' . SHOPPATH;
    ImportObjects('import.xml', ROOTPATH);

    return;
}

sub Cleanup {

    return if $ENV{'SKIP_CLEANUP'};

    my $System = LoadRootObject();
    if( $System->existsChild(ROOTOBJECT) ) {
        note 'deleting shop ' . SHOPPATH;
        $System->child(ROOTOBJECT)->delete();
    }

    return;
}

sub test {
    my $self = shift;

    $self->credentilas_validation_request_positive_test();
    $self->credentilas_validation_request_negative_test();
}

#========================================================================================
# Test methods
#========================================================================================
sub test_xml_import {
    my $subTestName = (caller(0))[3];

    my $testImportOK = 0;

    my $Shop = LoadObjectByPath(SHOPPATH);
    $testImportOK = isa_ok($Shop, 'DE_EPAGES::Shop::API::Object::Shop', "$subTestName - test shop found");

    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/DHLIntraship');
    $testImportOK = isa_ok($ShippingFulfillmentMethod,
            'DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship',
            "$subTestName - test ShippingFulfillmentMethodDHLIntraship found")
                        && $testImportOK;

    return $testImportOK;
}

sub credentilas_validation_request_positive_test {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $Shop = LoadObjectByPath(SHOPPATH);
    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/DHLIntraship');

    my $RemoteRESTDHL = DE_EPAGES::DHLIntraship::API::Object::RemoteRESTDHL->new();
    my $aAuthorizationData = [$ShippingFulfillmentMethod->get('WebserviceUsername'),$ShippingFulfillmentMethod->get('WebservicePassword')];

    my $CredentialsAreValid = $RemoteRESTDHL->validateCredentials(TEST_NATIONAL_ACCOUNT_NUMBER, $ShippingFulfillmentMethod, $aAuthorizationData, IS_TEST_MODE);
    is( $CredentialsAreValid, 1,
            "$subTestName - DHL Versenden credentials successfully validated");
}

sub credentilas_validation_request_negative_test {
    my $self = shift;
    my $subTestName = (caller(0))[3];

    my $Shop = LoadObjectByPath(SHOPPATH);
    my $ShippingFulfillmentMethod = $Shop->path('ShippingFulfillmentMethods/DHLIntraship');

    my $RemoteRESTDHL = DE_EPAGES::DHLIntraship::API::Object::RemoteRESTDHL->new();
    my $aAuthorizationData = [WRONG_USER,WRONG_PASSWORD];

    my $CredentialsAreValid = $RemoteRESTDHL->validateCredentials(TEST_NATIONAL_ACCOUNT_NUMBER, $ShippingFulfillmentMethod, $aAuthorizationData, IS_TEST_MODE);

    is( $CredentialsAreValid, 0,
            "$subTestName - DHL Versenden credentials are not valid");
}


RunScript(
    Sub => sub {
        RunOnStore( 'Store' => GetTestSite(), Sub => sub {
            Init();
            unless ( test_xml_import() ) {
                done_testing();
                exit;
            }

            SKIP: {
                skip 'external services tests' if $ENV{'SKIP_EXTERNAL_TESTS'};
                my $self = __PACKAGE__;
                $self->test();
            }
            done_testing();
            Cleanup();
        });
    }
);

1;