#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Address
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Address module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Address;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Address;

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

#use constant TEST_ADDRESS_COMPANY => 'My Company Name';
use constant TEST_ADDRESS_STREET => 'Diagonal 1';
use constant TEST_ADDRESS_STREET_TOO_LONG => 'street name that exceeds the maximum characters limit';
use constant TEST_ADDRESS_HOUSE_NUMBER => '435';
use constant TEST_ADDRESS_HOUSE_NUMBER_TOO_LONG => '43545678369521';
use constant TEST_ADDRESS_ADDRESS_ADDITION => '5th floor, 2nd door';
use constant TEST_ADDRESS_ZIP_CODE => '08036';
use constant TEST_ADDRESS_CITY => 'Barcelona';
use constant TEST_ADDRESS_CITY_TOO_LONG => 'city name that exceeds the maximum characters limit';
use constant TEST_ADDRESS_FULL_NAME => 'My Full Name';


#========================================================================================
sub test {
    my $self = shift;

    $self->address_shipper_assemble_positive_test();
    $self->address_receiver_assemble_positive_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub address_shipper_assemble_positive_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress();
    my $AddressWithWarnings = 0;
    my $mockedAddressElement = $self->_createAddressElement($AddressWithWarnings);
    my $hAddress = $mockedAddressElement->assemble($mockedAddress);

    is_deeply($hAddress,
        {
              'addressStreet' => TEST_ADDRESS_STREET,
              'addressAddition' => TEST_ADDRESS_ADDRESS_ADDITION,
              'postalCode' => TEST_ADDRESS_ZIP_CODE,
              'city' => TEST_ADDRESS_CITY,
        },
        "$SubTestName - address hash assembled"
    );
}

sub address_receiver_assemble_positive_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress();
    my $AddressWithWarnings = 0;
    my $mockedAddressElement = $self->_createAddressElement($AddressWithWarnings);
    my $hAddress = $mockedAddressElement->assemble($mockedAddress, TEST_ADDRESS_FULL_NAME);

    is_deeply($hAddress,
        {
          'addressAddition' => TEST_ADDRESS_ADDRESS_ADDITION,
          'city' => TEST_ADDRESS_CITY,
          'name2' => TEST_ADDRESS_FULL_NAME,
          'addressStreet' => TEST_ADDRESS_STREET,
          'postalCode' => TEST_ADDRESS_ZIP_CODE,
        },
        "$SubTestName - address hash assembled"
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createAddressElement {
    my $self = shift;
    my ($withWarning) = @_;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Address',
        $aParameterList
    );

    my $hOriginReturn = { };
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleOrigin', $hOriginReturn, [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' } ]
    );


    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($hAddress) = @_;

    $hAddress = {} unless defined($hAddress);

    my $hParameterList = {
        'Street' => $hAddress->{'Street'} // TEST_ADDRESS_STREET,
        'HouseNumber' => $hAddress->{'HouseNumber'} // TEST_ADDRESS_HOUSE_NUMBER,
        'Street2' => $hAddress->{'Street2'} // TEST_ADDRESS_ADDRESS_ADDITION,
        'Zipcode' => $hAddress->{'Zipcode'} // TEST_ADDRESS_ZIP_CODE,
        'City' => $hAddress->{'City'} // TEST_ADDRESS_CITY
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            $hParameterList
    );

    return $MockAddress->getMockObject();
}

__PACKAGE__->runScript;
