#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Name
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Name module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Name;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Name;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

use constant TEST_MERCHANT_NAME => 'John Doe';
use constant TEST_MERCHANT_NAME_TOO_LONG => 'John-Benjamin-Bertrand-Albert Doubtfire The Second Junior';
use constant TEST_COMPANY_NAME => 'Early Bird Manufacturer';
use constant TEST_COMPANY_NAME_TOO_LONG => 'Early Bird Manufacturer & Fruits collector from the South of the Country';

#========================================================================================
sub test {
    my $self = shift;

    $self->name_assemble_full_name_and_company_positive_test();
    $self->name_assemble_no_company_positive_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub name_assemble_full_name_and_company_positive_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress(
            { 'Company' => TEST_COMPANY_NAME, 'FullName' => TEST_MERCHANT_NAME }
    );
    my $hName = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Name->assemble($mockedAddress);

    is_deeply($hName,
        {
              'name1' => TEST_COMPANY_NAME,
              'name2' => TEST_MERCHANT_NAME,
        },
        "$SubTestName - Name hash assembled"
    );
}

sub name_assemble_no_company_positive_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedAddress = $self->_createAddress(
            { 'FullName' => TEST_MERCHANT_NAME  }
    );
    my $hName = DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Name->assemble($mockedAddress);

    is_deeply($hName,
        {
              'name1' => TEST_MERCHANT_NAME,
        },
        "$SubTestName - Name hash assembled"
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createAddress {
    my $self = shift;
    my ($hAddress) = @_;

    my $hParameterList = {
        'Company' => $hAddress->{'Company'},
        'FullName' => $hAddress->{'FullName'}
    };

    my $MockAddress = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            $hParameterList
    );

    return $MockAddress->getMockObject();
}


__PACKAGE__->runScript;
