#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Receiver
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Receiver module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Receiver;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

use constant TEST_ADDRESS_FULL_NAME => 'John Doe';
use constant TEST_ADDRESS_COMPANY_NAME => 'My Company Name';


#========================================================================================
sub test {
    my $self = shift;

    $self->receiver_assemble_with_address_test();
    $self->receiver_assemble_with_address_no_company_test();
   # $self->receiver_assemble_with_address_subnodes_with_warnings_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub receiver_assemble_with_address_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAddress = {
        'Company' => TEST_ADDRESS_COMPANY_NAME,
        'Country' => {
            'Code2' => 'DE',
        }
    };
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship( $hAddress );
    my $mockedReceiverElement = $self->_createReceiver($hAddress);
    my $hReceiver = $mockedReceiverElement->assemble($mockedShippingLabel);

    is_deeply($hReceiver,
        { 'consignee' => {
              'name1' => TEST_ADDRESS_COMPANY_NAME,
          }
        },
        "$SubTestName - Receiver hash assembled with an Address sub-node"
    );
}
sub receiver_assemble_with_address_no_company_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAddress = {
        'Company' => undef,
        'FullName' => TEST_ADDRESS_FULL_NAME
    };
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship( $hAddress );
    my $ReceiverWithWarnings = 0;
    my $mockedReceiverElement = $self->_createReceiver($ReceiverWithWarnings);
    my $hReceiver = $mockedReceiverElement->assemble($mockedShippingLabel);

    is_deeply($hReceiver,
        { 'consignee' => {
            'name1' => TEST_ADDRESS_FULL_NAME,
            }
        },
        "$SubTestName - Receiver hash assembled with an Address sub-node"
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createReceiver {
    my $self = shift;
    my ($withWarning) = @_;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Receiver',
        $aParameterList
    );

    my $hCommunicationReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleCommunication',
            $hCommunicationReturn,
            [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' } ]
    );

    my $hAddressReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleAddressReceiver',
            $hAddressReturn,
            [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' }, 'string' ]
    );

    return $Mockify->getMockObject();
}

sub _createShippingLabelDHLIntraship {
    my $self = shift;
    my ($hAddress) = @_;

    my $aParameterList = {
        'Address' => $self->_createAddress($hAddress),
    };

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($hAddress) = @_;

    my $hParameterList = {
        'Company' => $hAddress->{'Company'},
        'FullName' => $hAddress->{'FullName'},
        'Country' => {
            'Code2' => 'DE',
        }
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Address::API::Object::Address',
            $hParameterList
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript;
