#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Shipment
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Shipment module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Shipment;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;

use strict;

use constant TEST_SEQUENCE_NUMBER => 1234;
use constant TEST_LABEL_RESPONSE_TYPE => 'URL';

#========================================================================================
sub test {
    my $self = shift;

    $self->shipment_assemble_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub shipment_assemble_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship();
    my $ShipmentWithWarnings = 0;
    my $mockedShipmentElement = $self->_createShipment($ShipmentWithWarnings);
    my $hShipment = $mockedShipmentElement->assemble($mockedShippingLabel);

    is_deeply($hShipment,
        { 'shipments' =>[{}]
        },
        "$SubTestName - Shipment hash assembled");
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createShipment {
    my $self = shift;
    my ($withWarning) = @_;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipment',
        $aParameterList
    );

    my $hShipmentDetailsReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleShipmentDetails',
            $hShipmentDetailsReturn,
            [ { 'object' => 'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship' } ]
     );

    my $hShipperReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleShipper',
            $hShipperReturn,
            [ { 'object' => 'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship' } ]
    );

    my $hReceiverReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleReceiver',
            $hReceiverReturn,
            [ { 'object' => 'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship' } ]
    );

    return $Mockify->getMockObject();
}

sub _createShippingLabelDHLIntraship {
    my $self = shift;

    my $hParameterList = {};
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $hParameterList
    );

    return $Mockify->getMockObject();
}



__PACKAGE__->runScript;
