#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_ShipmentDetails
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for ShipmentDetails module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_ShipmentDetails;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify;
use Test::More;
use DateTime;

use strict;

use constant TEST_EKP => '5000000000';
use constant TEST_NATIONAL_ACCOUNT_NUMBER => TEST_EKP . '0101';
use constant TEST_WORLD_ACCOUNT_NUMBER => TEST_EKP . '5302';
use constant TEST_DHL_NATIONAL_PACKET_CODE => 'V01PAK';
use constant TEST_DHL_WORLD_PACKET_CODE => 'V53WPAK';
use constant TEST_SHIPPING_LABEL_ALIAS => '1019-1-00';
use constant TEST_ESTIMATED_DELIVERY_DATE_FORMATTED => '2017-04-25';
use constant TEST_ESTIMATED_DELIVERY_DATE_RAW => {
    year => 2017, month => 4, day => 25, hour => 16, minute => 12, second => 47, time_zone => 'Europe/Berlin'
};
use constant GERMANY_NATIONAL_COUNTRY_ID => 276;
use constant SPAIN_INTERNATIONAL_COUNTRY_ID => 724;

#========================================================================================
sub test {
    my $self = shift;

    $self->shipmentDetails_assemble_national_test();
    $self->shipmentDetails_assemble_international_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub shipmentDetails_assemble_national_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $nationalShipment = 1;
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship($nationalShipment);




    my $ShipmentDetailsWithWarnings = 0;
    my $mockedShipmentDetailsElement = $self->_createShipmentDetails($ShipmentDetailsWithWarnings);
    my $hShipmentDetails = $mockedShipmentDetailsElement->assemble($mockedShippingLabel);

    is_deeply($hShipmentDetails,
        {
            'billingNumber' => TEST_NATIONAL_ACCOUNT_NUMBER,
            'product' => TEST_DHL_NATIONAL_PACKET_CODE,
            'refNo' => TEST_SHIPPING_LABEL_ALIAS,
            'shipDate' => TEST_ESTIMATED_DELIVERY_DATE_FORMATTED
        },
        "$SubTestName - ShipmentDetails for national shipment hash assembled");
}

sub shipmentDetails_assemble_international_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $nationalShipment = 0;
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship($nationalShipment);

    my $ShipmentDetailsWithWarnings = 0;
    my $mockedShipmentDetailsElement = $self->_createShipmentDetails($ShipmentDetailsWithWarnings);
    my $hShipmentDetails = $mockedShipmentDetailsElement->assemble($mockedShippingLabel);

    is_deeply($hShipmentDetails,
        {
              'product' => TEST_DHL_WORLD_PACKET_CODE,
              'billingNumber' => TEST_WORLD_ACCOUNT_NUMBER,
              'refNo' => TEST_SHIPPING_LABEL_ALIAS,
              'shipDate' => TEST_ESTIMATED_DELIVERY_DATE_FORMATTED,
        },
        "$SubTestName - ShipmentDetails for international shipment hash assembled");
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createShipmentDetails {
    my $self = shift;
    my ($withWarning) = @_;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::ShipmentDetails',
        $aParameterList
    );

    my $hShipmentItemReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleShipmentItem',
            $hShipmentItemReturn,
            [ { 'object' => 'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship' } ]
    );

    return $Mockify->getMockObject();
}

sub _createShippingLabelDHLIntraship {
    my $self = shift;
    my ($nationalShipment) = @_;

    my $aParameterList = [
        'Alias' => TEST_SHIPPING_LABEL_ALIAS,
        'EstimatedDeliveryDate' => DateTime->new(TEST_ESTIMATED_DELIVERY_DATE_RAW),
        'Address' => $self->_createAddress($nationalShipment),
        'ShippingFulfillmentMethod' => $self->_createShippingFulfillmentMethodDHLIntraship()
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createShippingFulfillmentMethodDHLIntraship {
    my $self = shift;

    my $aParameterList = [
        'EKP' => TEST_EKP,
        'TeilnehmerNational' => '01',
        'TeilnehmerWeltpaket' => '02'
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;
    my ($nationalShipment) = @_;

    my $aParameterList = [
        'CountryID' => $nationalShipment ? GERMANY_NATIONAL_COUNTRY_ID : SPAIN_INTERNATIONAL_COUNTRY_ID,
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::Address::API::Object::Address',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript;
