#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_ShipmentItem
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for ShipmentItem module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_ShipmentItem;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

use strict;

use constant TEST_WEIGHT_IN_KG => 2.5194;
use constant TEST_ROUNDED_WEIGHT_IN_KG => 2.519;

use constant TEST_OVERWEIGHT_IN_KG => 122;
use constant TEST_UNDERWEIGHT_IN_KG => 0.0001;
use constant TEST_LENGHT_IN_MM => 155;
use constant TEST_LENGHT_IN_CM => 15;
use constant TEST_WIDTH_IN_MM => 200;
use constant TEST_WIDTH_IN_CM => 20;
use constant TEST_HEIGHT_IN_MM => 53;
use constant TEST_HEIGHT_IN_CM => 5;
use constant CENTIMETRE_TO_METER_FACTOR => 0.01;
use constant MILIMETRE_TO_METER_FACTOR => 0.001;


#========================================================================================
sub test {
    my $self = shift;

    $self->shipmentItem_assemble_no_warnings_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub shipmentItem_assemble_no_warnings_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship(
        {'TotalWeight' => TEST_WEIGHT_IN_KG}
    );
    my $mockedShipmentItemElement = $self->_createShipmentItem();
    my $hShipmentItem = $mockedShipmentItemElement->assemble($mockedShippingLabel);

    is_deeply($hShipmentItem,
        { 'details' => {
                'weight' => { 'uom'   => 'kg',
                    'value' => TEST_ROUNDED_WEIGHT_IN_KG
                },
                'dim'   => {
                    'uom'       => 'cm',
                    'height'    => TEST_HEIGHT_IN_CM,
                    'length'    => TEST_LENGHT_IN_CM,
                    'width'     => TEST_WIDTH_IN_CM
                }
            }
        },
        "$SubTestName - ShipmentItem hash assembled"
    );
}

sub shipmentItem_assemble_no_parcel_dimensions_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedShippingLabelNoDimensions =
        $self->_createShippingLabelDHLIntraship({ 'ParcelDepth' => '', 'ParcelLength' => '', 'ParcelWidth' => '' });
    my $mockedShipmentItemElement = $self->_createShipmentItem();
    my $hShipmentItem = $mockedShipmentItemElement->assemble($mockedShippingLabelNoDimensions);

    is_deeply($hShipmentItem,
        { 'details' => {
                'weight' => { 'uom' => 'kg',
                    'value'         => TEST_ROUNDED_WEIGHT_IN_KG
                },
                'dim'    => {
                    'uom'    => 'cm',
                    'height' => 0,
                    'length' => 0,
                    'width'  => 0
                }
            }
        },
        "$SubTestName - ShipmentItem hash assembled having no dimensions"
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createShipmentItem {
    my $self = shift;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::ShipmentItem',
        $aParameterList
    );
    $Mockify->addMock('_getCentimetreFactor', sub { return CENTIMETRE_TO_METER_FACTOR });

    return $Mockify->getMockObject();
}

sub _createShippingLabelDHLIntraship {
    my $self = shift;
    my ($hParameterList) = @_;

    my $aParameterList = [
        'ParcelDepth' => $hParameterList->{'ParcelDepth'} // TEST_HEIGHT_IN_MM,
        'ParcelLength' => $hParameterList->{'ParcelLength'} // TEST_LENGHT_IN_MM,
        'ParcelWidth' => $hParameterList->{'ParcelWidth'} // TEST_WIDTH_IN_MM,
        'SizeUnit' => $self->_createUnitOfMeasurementForMilimetre(),
        'TotalWeight' => $hParameterList->{'TotalWeight'} // TEST_WEIGHT_IN_KG,
    ];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createUnitOfMeasurementForMilimetre {
    my $self = shift;

    my $aParameterList = [
        'Factor' => MILIMETRE_TO_METER_FACTOR,
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript;
