#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_ShipmentOrder
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for ShipmentOrder module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_ShipmentOrder;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::More;
use Data::Dumper;
use strict;

use constant TEST_SEQUENCE_NUMBER => '1234-1';
use constant TEST_SEQUENCE_NUMBER_TOO_LONG => 'That-Sequence-Number-Is-that_exceeds_the_maximum_characters_limit';
use constant TEST_LABEL_RESPONSE_TYPE => 'URL';

#========================================================================================
sub test {
    my $self = shift;

    $self->shipmentOrder_assemble_positive_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub shipmentOrder_assemble_positive_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $sequenceNumberTooLong = 0;
    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship($sequenceNumberTooLong);

    my $ShipmentOrderWithWarning = 0;
    my $mockedShipmentOrderElement = $self->_createShipmentOrder($ShipmentOrderWithWarning);
    my $hShipmentOrder = $mockedShipmentOrderElement->assemble($mockedShippingLabel);

    is_deeply($hShipmentOrder,
        { 'profile' => 'STANDARD_GRUPPENPROFIL'},
        "$SubTestName - ShipmentOrder array (with a single hash) assembled");
}


#========================================================================================
# Mocking methods
#========================================================================================
sub _createShipmentOrder {
    my $self = shift;
    my ($withWarning) = @_;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::ShipmentOrder',
        $aParameterList
    );

    my $hShipmentReturn = { 'profile' => 'STANDARD_GRUPPENPROFIL'};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleShipment',
            $hShipmentReturn,
             [ { 'object' => 'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship' } ]
    );

    return $Mockify->getMockObject();
}

sub _createShippingLabelDHLIntraship {
    my $self = shift;
    my ($sequenceNumberTooLong) = @_;

    my $hParameterList = {
        'ID' => $sequenceNumberTooLong ? TEST_SEQUENCE_NUMBER_TOO_LONG : TEST_SEQUENCE_NUMBER,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $hParameterList
    );

    $Mockify->addMock('id', sub {
        my $self = shift;
        return $self->get('ID');
    });

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript;
