#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Shipper
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for Shipper module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::ShippingLabelRequest_Shipper;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::Mockify;
use Test::More;

use strict;

#========================================================================================
sub test {
    my $self = shift;

    $self->shipper_assemble_test();

    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub shipper_assemble_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedShippingLabel = $self->_createShippingLabelDHLIntraship();
    my $ShipperWithWarnings = 0;
    my $mockedShipperElement = $self->_createShipper($ShipperWithWarnings);
    my $hShipper = $mockedShipperElement->assemble($mockedShippingLabel);

    is_deeply($hShipper,
        { 'shipper' => {}
        },
        "$SubTestName - Shipper hash assembled"
    );
}

#========================================================================================
# Mocking methods
#========================================================================================
sub _createShipper {
    my $self = shift;
    my ($withWarning) = @_;

    my $aParameterList = [];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::DHLIntraship::API::REST::ShippingLabelRequest::Shipper',
        $aParameterList
    );

    my $hNameReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleName',
            $hNameReturn,
            [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' } ]
    );

    my $hAddressReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleAddress',
            $hAddressReturn,
            [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' } ]
    );

    my $hCommunicationReturn = {};
    $Mockify->addMockWithReturnValueAndParameterCheck(
            '_assembleCommunication',
            $hCommunicationReturn,
            [ { 'object' => 'DE_EPAGES::Address::API::Object::Address' } ]
    );

    return $Mockify->getMockObject();
}

sub _createShippingLabelDHLIntraship {
    my $self = shift;

    my $aParameterList = [
        'Order' => $self->_createOrder(),
    ];

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createOrder {
    my $self = shift;

    my $aParameterList = [
        'ShopAddress' => $self->_createAddress(),
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

sub _createAddress {
    my $self = shift;

    my $aParameterList = [];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::Address::API::Object::Address',
            $aParameterList
    );

    return $Mockify->getMockObject();
}

__PACKAGE__->runScript;
