#========================================================================================
# package      DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay
# state        public
# base         DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal
# description  object interface for LineItemPaymentDocdataPay.
#========================================================================================
package DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay;
use base DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal;

use strict;
use XML::Simple;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError);
use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime isValid );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::DocdataPay::API::Constants qw (
  LATEST_API_VERSION
  OLD_API_VERSION
  %TRANSACTION_STATUS_0_4
  %TRANSACTION_STATUS_1_2
  $VALID_PAYABLE_STATUS
  $FAILED_PAYABLE_STATUS
  ORDER_STATUS_PAID
);
use DE_EPAGES::DocdataPay::API::Object::RemoteDocdataPay;
use DE_EPAGES::DocdataPay::API::SOAP::CommunicationHandler;
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByGUID );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
sub ErrorDefinition {
    return { 'UnknownStatusValue' => 'Unknown Status Value: #StatusValue',
             'InvalidXMLError' => 'XML Structure is no valid'
    };
}

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $LineItemPaymentDocdataPay->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className { 'LineItemPaymentDocdataPay' }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentDocdataPay = DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemPaymentDocdataPay');
#               my $LineItemPaymentDocdataPay = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemPaymentDocdataPay' is triggered.
#----------------------------------------------------------------------------------------
# input        $Package | class package | string
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemPaymentDocdataPay | new object | object
# hook         OBJ_InsertLineItemPaymentDocdataPay | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentDocdataPayID - line item payment id - int
#               </ul>
#========================================================================================
sub insert {
    my ( $Package, $hInfo ) = @_;

    my $self = $Package->SUPER::insert($hInfo);

    TriggerHook( 'OBJ_InsertLineItemPaymentDocdataPay',
        { 'Object' => $self, 'LineItemPaymentDocdataPayID' => $self->id } );
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentDocdataPay->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemPaymentDocdataPay' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemPaymentDocdataPay | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentDocdataPayID - line item payment id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook( 'OBJ_DeleteLineItemPaymentDocdataPay',
        { 'Object' => $self, 'LineItemPaymentDocdataPayID' => $self->id } );
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentDocdataPay->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItemPaymentDocdataPay' and 'OBJ_AfterUpdateLineItemPaymentDocdataPay'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItemPaymentDocdataPay | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentDocdataPayID - line item payment id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItemPaymentDocdataPay | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentDocdataPayID - line item payment id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ( $hValues, $LanguageID ) = @_;

    TriggerHook(
        'OBJ_BeforeUpdateLineItemPaymentDocdataPay',
        {
            'Object'                      => $self,
            'Values'                      => $hValues,
            'LineItemPaymentDocdataPayID' => $self->id,
            'LanguageID'                  => $LanguageID
        }
    );
    $self->SUPER::set( $hValues, $LanguageID );
    TriggerHook(
        'OBJ_AfterUpdateLineItemPaymentDocdataPay',
        {
            'Object'                      => $self,
            'Values'                      => $hValues,
            'LineItemPaymentDocdataPayID' => $self->id,
            'LanguageID'                  => $LanguageID
        }
    );
    return;
}

#========================================================================================
# function     getTransStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Object->getTransStatus( $Input1, $Input2);
# example      ${ReturnVariable} = $Object->getTransStatus( Example1, Example2 );
#----------------------------------------------------------------------------------------
# description  Fetches Transaction Status from Docdata
#----------------------------------------------------------------------------------------
# return       $LocalStatus | string representation of transaction status code. Possible
#               values are: STATUS_STARTED, STATUS_PAID, STATUS_CLOSED, STATUS_FAILED
#               | string
#========================================================================================
sub getTransStatus {
    my $self = shift;
    $self->_TestObject(
        'PaymentMethodDocdataPay',
        $self->get('PaymentMethod'),
        'DE_EPAGES::DocdataPay::API::Object::PaymentMethodDocdataPay'
    );

    if ( $self->get('APIVersion') eq OLD_API_VERSION ) {
        return $self->_getTransStatus04();
    }
    return $self->_getTransStatus12( $self->get('IsTestTransaction') );
}

#========================================================================================
# function     _getTransStatus12
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hTransStatus = $self->_getTransStatus12($IsInTestMode);
#----------------------------------------------------------------------------------------
# description  Runs status request and parses its response.
#----------------------------------------------------------------------------------------
# input       $IsInTestMode              | True if it should run on test mode | Boolean
# return      $hTransStatus | hash with Calculated order status and payment method string | hash ref
#========================================================================================
sub _getTransStatus12 {
    my $self = shift;
    my ($IsInTestMode) = @_;

    my $Order = $self->parent()->parent();
    my ( $hResponse, $hTrace ) =
      $self->_sendRequest( 'status', $IsInTestMode, $Order );

    my $OrderStatus = $self->_calculatePaymentStatus( $hResponse );

    if($OrderStatus eq ORDER_STATUS_PAID) {
        $self->_updateTransactionStatus( $TRANSACTION_STATUS_1_2{$OrderStatus} );
    } else {
        $self->_updateTransactionStatus( $TRANSACTION_STATUS_1_2{ $hResponse->{'authorizationStatus'} } );
    }

    my $PaymentMethod = $self->_getPaymentMethod($hTrace);

    return ( $OrderStatus, $PaymentMethod );
}

#========================================================================================
# function     _calculatePaymentStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       _calculatePaymentStatus( $hResponse );
#----------------------------------------------------------------------------------------
# description  It determines when an order should be mark as PAID or not
#----------------------------------------------------------------------------------------
# input        $hResponse | Response from status status request | ref_hash
# return       $OrderStatus | Calculated order status | String
#========================================================================================
sub _calculatePaymentStatus {
    my $self = shift;
    my ($hResponse) = @_;

    my $OrderStatus;

    if ( ( $hResponse->{'totalCaptured'} eq $hResponse->{'totalRegistered'} )
        || $hResponse->{'totalAcquirerApproved'} eq
        $hResponse->{'totalRegistered'} )
    {
        $OrderStatus = ORDER_STATUS_PAID;
    }

    return $OrderStatus;
}

#========================================================================================
# function     _updateTransactionStatus
# state        private
#----------------------------------------------------------------------------------------
# syntax       _updateTransactionStatus($Status);
#----------------------------------------------------------------------------------------
# description  Updates LineItemPaymentDocdataPay's transaction status.
#----------------------------------------------------------------------------------------
# input       $Status              | New transaction status | String
#========================================================================================
sub _updateTransactionStatus {
    my $self = shift;
    my ($Status) = @_;

    $self->set( { 'TransStatus'  => $Status } );
    $self->set( { 'TransTimeMod' => $self->_getCurrentDateTime() } );
    unless ( $self->get('TransTime') ) {
        $self->set( { 'TransTime' => $self->get('TransTimeMod') } );
    }

    return;
}

#========================================================================================
# function     _getPaymentMethod
# state        private
#----------------------------------------------------------------------------------------
# syntax       $pmethod = $self->_getPaymentMethod($Trace);
#----------------------------------------------------------------------------------------
# description  Runs status request and parses its response.
#----------------------------------------------------------------------------------------
# input       $Trace | response trace object | Object
# return      $pmethod | payment method string | String
#========================================================================================
sub _getPaymentMethod {
    my $self = shift;
    my ($hTrace) = @_;

    my $xs = XML::Simple->new( );
    my $XML;

    eval {
        $XML = $xs->XMLin( $hTrace->{'http_response'}->{'_content'} );
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error('InvalidXMLError',$Error->message());
    }

    return $XML->{'S:Body'}->{'statusResponse'}->{'statusSuccess'}->{'report'}->{'payment'}->{'paymentMethod'};
}

#========================================================================================
# function     setOrderStatus
# state        public
#----------------------------------------------------------------------------------------
# syntax       setOrderStatus($Order, $OrderStatusParameter);
#----------------------------------------------------------------------------------------
# description  Sets an Order to paid.
#----------------------------------------------------------------------------------------
# input       $Order | The Order | Object
# input       $OrderStatusParameter | The order status parameter | String
#========================================================================================
sub setOrderStatus {
    my $self = shift;
    my ($OrderStatusParameter) = @_;

    my $Order = $self->parent()->parent();

    unless ( $self->_isValid( $Order->get($OrderStatusParameter) ) ) {
        $Order->set( { $OrderStatusParameter => $self->get('TransTime') } );
    }

    return;
}

#========================================================================================
sub _getTransStatus04 {
    my $self = shift;

    my $Remote =
      DE_EPAGES::DocdataPay::API::Object::RemoteDocdataPay->factory($self);

    my $Hash = $self->get('Field1');
    return unless ( defined $Hash );

    my $Status =
      $Remote->statusPaymentCluster( { 'payment_cluster_key' => $Hash },
        $self->get('GUID') );
    chomp($Status);

    my $LocalStatus = $TRANSACTION_STATUS_0_4{$Status};
    unless ( defined($LocalStatus) ) {
        Error( 'UnknownStatusValue', { 'Message' => $Status } );
    }

    $self->_updateTransactionStatus($LocalStatus);

    return $LocalStatus;
}

#========================================================================================
# function     capturePayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       capturePayment();
#----------------------------------------------------------------------------------------
# description  Sends Capture Request to DocdataPay.
#========================================================================================
sub capturePayment {
    my $self = shift;

    return unless ( defined $self->get('PaymentID') );
    return
      unless ( defined $self->get('APIVersion')
        && $self->get('APIVersion') eq LATEST_API_VERSION );

    my $IsInTestMode = $self->get('IsTestTransaction');
    my $Order        = $self->parent()->parent();
    $self->_TestObject( 'CustomerOrder', $Order,
        'DE_EPAGES::Order::API::Object::CustomerOrder' );
    $self->_sendRequest( 'capture', $IsInTestMode, $Order );

    return;
}

#========================================================================================
# function     executePayment
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentDocdataPay->executePayment($hParams);
#----------------------------------------------------------------------------------------
# description  This function directs the payment process.
#----------------------------------------------------------------------------------------
# input        $hValues | environment information
#                       <ul>
#                           <li>Basket - basket object (optional)</li>
#                           <li>Servlet - servlet handling the current request</li>
#                           <li>Order - order object (optional)</li>
#                       </ul>
#                   | ref.hash
#========================================================================================
sub executePayment {
    my $self = shift;
    my ($hParams) = @_;

    $self->updateOnExecute($hParams);

    my $ViewAction =
      $hParams->{'Servlet'}->vars('IsMobile')
      ? 'MobileSF-ViewPaymentDocdataPay'
      : 'ViewPaymentDocdataPay';

    # show payment template
    my $Object = $hParams->{'Order'} // $hParams->{'Basket'};
    Error(
        'INVALID_FORM',
        {
            'ViewVars' => {
                'FormError'    => 0,
                'ViewObjectID' => $Object->id,
            },
            'ViewAction' => $ViewAction
        }
    );

    return;
}

#========================================================================================
# function     getDisplayTransID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DisplayTransID = $LineItemPayment->getDisplayTransID() ;
#----------------------------------------------------------------------------------------
# description  This overwrites the default LineItemPaymentExternal->getDisplayTransID()
#               method to return TransID2 instead the default TransID.
#----------------------------------------------------------------------------------------
# return       $DisplayTransID | TransID2 attribute | string
#========================================================================================
sub getDisplayTransID {
    my $self = shift;

    return $self->get('TransID');
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::DocdataPay::API::Object::LineItemPaymentDocdataPay->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ( $Parent, $hInfo ) = @_;

    my $Shop = $Parent->getSite();
    my $PaymentMethodDocdataPay = $hInfo->{'PaymentMethod'} ? $class->_LoadObjectByGUID( $hInfo->{'PaymentMethod'}->{'Attributes'}->{'GUID'} ) : undef;

    return {
        %{ $class->SUPER::default( $Parent, $hInfo ) },
        'APIVersion'        => $PaymentMethodDocdataPay ? $PaymentMethodDocdataPay->get('APIVersion') : LATEST_API_VERSION,
        'IsTestTransaction' => $Shop->get('IsInternalTestShop'),
    };
}

#========================================================================================
# function     updateOnExecute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $updateOnExecute($hParams);
#----------------------------------------------------------------------------------------
# description  This functions sets amount and currency.
#----------------------------------------------------------------------------------------
# input        $hValues | environment information
#                       <ul>
#                           <li>Basket - basket object</li>
#                           <li>Servlet - servlet handling the current request</li>
#                           <li>Order - order object (optional)</li>
#                       </ul>
#                   | ref.hash
#========================================================================================
sub updateOnExecute {
    my $self = shift;
    my ($hParams)= @_;

    # Docdata don't use IsTestMode flag from payment method, hence we keep what we get up to this point.
    my $IsTestTransaction = $self->get('IsTestTransaction');

    $self->SUPER::updateOnExecute($hParams);

    $self->set({'IsTestTransaction' => $IsTestTransaction});

    return;
}


# Wrappers
#----------------------------------------------------------------------------------------
sub _getCurrentDateTime {
    my $self = shift;

    return GetCurrentDateTime();
}

#----------------------------------------------------------------------------------------
sub _isValid {
    my $self = shift;
    my ($Object) = @_;

    return isValid($Object);
}

#----------------------------------------------------------------------------------------
sub _sendRequest {
    my $self = shift;
    my ( $RequestType, $IsInTestMode, $Order ) = @_;

    my $CommunicationHandler =
      DE_EPAGES::DocdataPay::API::SOAP::CommunicationHandler->new(
        $IsInTestMode);

    return $CommunicationHandler->sendRequest( $RequestType, $Order );
}

#----------------------------------------------------------------------------------------
sub _TestObject {
    my $self = shift;
    my ( $ClassName, $Object, $Package ) = @_;

    return TestObject( $ClassName, $Object, $Package );
}
#----------------------------------------------------------------------------------------
sub _LoadObjectByGUID {
    my $self = shift;
    my ($GUID) = @_;

    return LoadObjectByGUID($GUID);
}

1;
