#========================================================================================
# §package      DE_EPAGES::MolliePayments::API::Constants
# §state        private
#----------------------------------------------------------------------------------------
# §description  constants for cartridge
#========================================================================================
package DE_EPAGES::MolliePayments::API::Constants;
use base Exporter;

use strict;

our @EXPORT_OK = qw (
    CREATE_LINKS_API_V2_URL
    INTERNAL_CARTRIDGE_NAME
    SUPPORTED_CURRENCIES
    @SPLIT_GATEWAY_METHODS
    $SPLIT_GATEWAY_METHODS_MAXROWS
    PAYMENT_TYPE_MOLLIE_PAYMENTS
    PAYMENT_METHOD_MOLLIE_PAYMENTS
    LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS
    SUPPORTED_LOCALES
    DEFAULT_LOCALE
    SUPPORTED_ORDER_EMBED
    ORDER_PARAM_EMBED_PAYMENTS
    SUPPORTED_MOLLIE_PAYMENT_STATUS
    TEMPORARY_MOLLIE_PAYMENT_STATUS
    DEFINITIVE_MOLLIE_PAYMENT_STATUS
    MOLLIE_PAYMENT_STATUS_ERRORS
    PAYMENT_STATUS_AUTHORIZED
    PAYMENT_STATUS_CANCELED
    PAYMENT_STATUS_EXPIRED
    PAYMENT_STATUS_FAILED
    PAYMENT_STATUS_PAID
    PAYMENT_STATUS_PENDING
    MOLLIE_ORDER_STATUS_CANCELED
    PROFILES_LIMIT_MOLLIE_PAYMENTS
    METHODS_PARAM_ORDERS_RESOURCE
    METHODS_PARAM_INCLUDE_WALLETS
    APP_SCOPES
    ORDER_ERROR_UNSUPPORTED_CURRENCY
    ORDER_ERROR_UNSUPPORTED_COUNTRY
    REQUIRED_BILLING_ADDRESS_FIELDS
    @DISPLAY_PAYMENT_ICONS
    @MOLLIE_LIMITED_PAYMENTMETHODS
);

use constant CREATE_LINKS_API_V2_URL => 'https://api.mollie.com/v2/client-links';

use constant PROFILES_LIMIT_MOLLIE_PAYMENTS => 20;
use constant LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS => 'LineItemPaymentMolliePayments';
use constant PAYMENT_TYPE_MOLLIE_PAYMENTS => 'MolliePayments';
use constant PAYMENT_METHOD_MOLLIE_PAYMENTS => 'PaymentMethodMolliePayments';

use constant ORDER_PARAM_EMBED_PAYMENTS => 'payments';
use constant METHODS_PARAM_ORDERS_RESOURCE => 'orders';
use constant METHODS_PARAM_INCLUDE_WALLETS => 'applepay';

# App permissions https://docs.mollie.com/oauth/permissions
use constant APP_SCOPES => qw(
    profiles.read
    orders.read
    orders.write
    payments.read
    payments.write
    shipments.read
    shipments.write
    refunds.read
    refunds.write
);

use constant ORDER_ERROR_UNSUPPORTED_CURRENCY => 'The currency is not supported by this payment method';
use constant ORDER_ERROR_UNSUPPORTED_COUNTRY => 'The billing country is not supported for this payment method';

use constant SUPPORTED_MOLLIE_PAYMENT_STATUS => [qw(
    open
    pending
    authorized
    paid
    canceled
    expired
    failed
)];

use constant TEMPORARY_MOLLIE_PAYMENT_STATUS => [qw(
    open
    pending
    authorized
)];

use constant DEFINITIVE_MOLLIE_PAYMENT_STATUS => [qw(
    canceled
    paid
    expired
    failed
)];
#

use constant MOLLIE_PAYMENT_STATUS_ERRORS => [qw(
    canceled
    expired
    failed
)];

use constant PAYMENT_STATUS_AUTHORIZED => 'authorized';
use constant PAYMENT_STATUS_CANCELED => 'canceled';
use constant PAYMENT_STATUS_EXPIRED => 'expired';
use constant PAYMENT_STATUS_FAILED => 'failed';
use constant PAYMENT_STATUS_PAID => 'paid';
use constant PAYMENT_STATUS_PENDING => 'pending';

use constant MOLLIE_ORDER_STATUS_CANCELED => 'canceled';

# Fields which the merchant needs to set to mandatory in the registration settings
use constant REQUIRED_BILLING_ADDRESS_FIELDS => qw ( FirstName LastName City Street Zipcode CountryID EMail );

# This name is used to load objects from database (DON'T CHANGE!)
use constant INTERNAL_CARTRIDGE_NAME => 'MolliePayments';

# Currencies which are supported by MolliePayments.
use constant SUPPORTED_CURRENCIES => [qw(
    EUR
    GBP
    DKK
    NOK
    PLN
    SEK
    CHF
    USD
)];

# Order Embed which are supported by MolliePayments.
use constant SUPPORTED_ORDER_EMBED => [qw(
    payments
    shipments
)];

# Fallback Locale in case the one we have is not supported by MolliePayments
use constant DEFAULT_LOCALE => 'en_US';

# Locales that are supported by MolliePayments
use constant SUPPORTED_LOCALES => [qw(
    en_US
    nl_NL
    nl_BE
    fr_FR
    fr_BE
    de_DE
    de_AT
    de_CH
    es_ES
    ca_ES
    pt_PT
    it_IT
    nb_NO
    sv_SE
    fi_FI
    da_DK
    is_IS
    hu_HU
    pl_PL
    lv_LV
    lt_LT
)];

our @AllPaymentMethods = (
    #@CreditCards
    {
        'isGroup'  => 0,
        'ID'       => 'creditcard',
        'REAL_SGM' => 'creditcard',
        'Name'     => 'Credit Card',
        'LogoName' => 'CreditCard2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'giftcard',
        'REAL_SGM' => 'giftcard',
        'Name'     => 'Gift Card',
        'LogoName' => 'GiftCard2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'paysafecard',
        'REAL_SGM' => 'paysafecard',
        'Name'     => 'Pay Safe Card',
        'LogoName' => 'PaySafeCard2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'belfius',
        'REAL_SGM'      => 'belfius',
        'Name'          => 'Belfius Direct Net',
        'LogoName'      => 'Belfius2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'giropay',
        'REAL_SGM'      => 'giropay',
        'Name'          => 'Giropay',
        'LogoName'      => 'GiroPay2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'kbc',
        'REAL_SGM'      => 'kbc',
        'Name'          => 'KBC Online',
        'LogoName'      => 'KBC2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'ideal',
        'REAL_SGM'      => 'ideal',
        'Name'          => 'iDEAL',
        'LogoName'      => 'Ideal2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'sofort',
        'REAL_SGM'      => 'sofort',
        'Name'          => 'Sofort',
        'LogoName'      => 'Sofort2x',
    },
    {
        'isGroup'       => 0,
        'ID'            => 'inghomepay',
        'REAL_SGM'      => 'inghomepay',
        'Name'          => 'ING HomePay',
        'LogoName'      => 'INGHomePay2x'
    },
    #@DirectDebit
    {
        'isGroup'       => 0,
        'ID'            => 'directdebit',
        'REAL_SGM'      => 'directdebit',
        'Name'          => 'Direct Debit',
        'LogoName'      => 'DirectDebit2x'
    },
    #@OfflinePayByInvoice
    {
        'isGroup'       => 0,
        'ID'            => 'banktransfer',
        'REAL_SGM'      => 'banktransfer',
        'Name'          => 'Bank transfer',
        'LogoName'      => 'BankTransfer2x'
    },
    #@OtherPaymentMethods
    {
        'isGroup'       => 0,
        'ID'            => 'applepay',
        'REAL_SGM'      => 'applepay',
        'Name'          => 'ApplePay',
        'LogoName'      => 'ApplePay2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'bancontact',
        'REAL_SGM'      => 'bancontact',
        'Name'          => 'Bancontact',
        'LogoName'      => 'BanContact2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'eps',
        'REAL_SGM'      => 'eps',
        'Name'          => 'EPS',
        'LogoName'      => 'EPS2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'mybank',
        'REAL_SGM'      => 'mybank',
        'Name'          => 'MyBank',
        'LogoName'      => 'MyBank2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'paypal',
        'REAL_SGM'      => 'paypal',
        'Name'          => 'PayPal',
        'LogoName'      => 'PayPal2x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'przelewy24',
        'REAL_SGM'      => 'przelewy24',
        'Name'          => 'Przelewy24',
        'LogoName'      => 'Przelewy242x'
    },
    {
        'isGroup'       => 0,
        'ID'            => 'voucher',
        'REAL_SGM'      => 'voucher',
        'Name'          => 'Voucher',
        'LogoName'      => 'Voucher2x'
    },
    #@KlarnaPayments
    {
        'isGroup'  => 0,
        'ID'       => 'klarnapaylater',
        'REAL_SGM' => 'klarnapaylater',
        'Name'     => 'Klarna: Pay later',
        'LogoName' => 'Klarna2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'klarnasliceit',
        'REAL_SGM' => 'klarnasliceit',
        'Name'     => 'Klarna: Slice it',
        'LogoName' => 'Klarna2x'
    },
    {
        'isGroup'  => 0,
        'ID'       => 'klarnapaynow',
        'REAL_SGM' => 'klarnapaynow',
        'Name'     => 'Klarna: Pay Now',
        'LogoName' => 'Klarna2x'
    }
);

our @DISPLAY_PAYMENT_ICONS = (
    {
        'DisplayAlways' => 1,
        'ID'            => 'Klarna',
        'LocaleID'      => 'all',
        'Icon'          => 'klarna-pink.svg',
        'Name'          => 'Klarna'

    },
    {
        'DisplayAlways' => 1,
        'ID'            => 'PayPal',
        'LocaleID'      => 'all',
        'Icon'          => 'paypal-logo-small.svg',
        'Name'          => 'PayPal'
    },
    {
        'DisplayAlways' => 1,
        'ID'            => 'MasterCard',
        'LocaleID'      => 'all',
        'Icon'          => 'mastercard-logo.svg',
        'Name'          => 'MasterCard'
    },
    {
        'DisplayAlways' => 1,
        'ID'            => 'Visa',
        'LocaleID'      => 'all',
        'Icon'          => 'visa-logo.svg',
        'Name'          => 'Visa'
    },
    {
        'DisplayAlways' => 1,
        'ID'            => 'ApplePay',
        'LocaleID'      => 'all',
        'Icon'          => 'apple-pay.svg',
        'Name'          => 'ApplePay'
    },
    # have to clarify, if Google Pay is supported by Mollie
    #{
    #    'DisplayAlways' => 1,
    #    'ID'            => 'GooglePay',
    #    'LocaleID'      => 'all',
    #    'Icon'          => 'google-pay.svg',
    #    'Name'          => 'GooglePay'
    #},
    {
        'DisplayAlways' => 0,
        'ID'            => 'GiroPay',
        'LocaleID'      => 'de_DE',
        'Icon'          => 'giropay.svg',
        'Name'          => 'GiroPay'
    },
    {
        'DisplayAlways' => 0,
        'ID'            => 'Cartes Bancaires',
        'LocaleID'      => 'fr_FR',
        'Icon'          => 'cartes-bancaires-logo.svg',
        'Name'          => 'Cartes Bancaires'
    },
    {
        'DisplayAlways' => 0,
        'ID'            => 'eps',
        'LocaleID'      => 'de_AT',
        'Icon'          => 'eps.svg',
        'Name'          => 'EPS'
    },
    {
        'DisplayAlways' => 0,
        'ID'            => 'iDeal',
        'LocaleID'      => 'nl_NL',
        'Icon'          => 'ideal-logo.svg',
        'Name'          => 'iDeal'
    }
);

our @MOLLIE_LIMITED_PAYMENTMETHODS = (
    {
        'isGroup'  => 0,
        'ID'       => 'creditcard',
        'REAL_SGM' => 'creditcard',
        'Name'     => 'Credit Card',
        'LogoName' => 'CreditCard2x',
        'Icon'     => 'molliepayments-credit-card.png'
    }
    # for testing the restrictions
    #,{
    #     'isGroup'  => 0,
    #     'ID'       => 'klarnapaynow',
    #     'REAL_SGM' => 'klarnapaynow',
    #     'Name'     => 'Klarna: Pay Now',
    #     'LogoName' => 'Klarna2x',
    #     'Icon'     => 'klarna-logo.svg'
    # },
    # {
    #     'isGroup'       => 0,
    #     'ID'            => 'paypal',
    #     'REAL_SGM'      => 'paypal',
    #     'Name'          => 'PayPal',
    #     'LogoName'      => 'PayPal2x',
    #     'Icon'          => 'paypal-logo-small.svg'
    # }
);

our @SPLIT_GATEWAY_METHODS = (
    {
        'isGroup'  => 1,
        'ID'       => '_01',
        'Name'     => '',
        'GrpCount' => scalar(@AllPaymentMethods)
    },
    @AllPaymentMethods
);

our $SPLIT_GATEWAY_METHODS_MAXROWS = scalar(@AllPaymentMethods);

1;
